
(* :Title: Integration *)

(* :Author: Roger Germundsson, 1998-2000 *)

(* :Summary:
	Extends Integrate and NIntegrate to do definite integration over a region 
	described by a logical combination of algebraic inequalities, 
	i.e. semi-algebraic sets. 
	
	And extends Integrate to do integration of piecewise functions including 
	nested expressions involving: If, Which, UnitStep, Boole, Abs, Sign, 
	Min, Max, Floor, Ceiling, Round, IntegerPart, FractionalPart, Mod and
	Quotient.
	
	These two extensions can be used interchangeably, i.e. you can have 
	inequalities involving the various piecewise functions and you can 
	have piecewise functions using region inequalities.
 *)
 
(* :Context: Calculus`Integration` *)

(* :Package Version: 1.2 *)

(* :History:
	V0.9 Roger Germundsson, April 1998 
				- initial implementation inequality conversions
				- support for Abs/Min/Max in inequalities
				- Enqvist example gallery
	V1.0 Roger Germundsson, December 1998 
				- repackaging (as application package), robustness enhancements
				- use generic CAD when possible for performance 
				- support for Abs/Min/Max now built into CAD 
				- general solution for nested piecewise functions
				- added support for numeric integration over regions
				- user documentation
	V1.1 Roger Germundsson, July 1999 
				- repackaging now fail as other M-- functions, i.e. return unevaluated
				- added support for algebraic numbers and functions in inequalities
				- added experimental support for regions containing parameters
				- complete Enqvist example gallery
				- partial Demidovich example gallery
	V1.2 Roger Germundsson, Feb 2000
				- added support for finite piecewise functions (Abs/Min/Max/Sign)
				  as piecewise greatly extended integration capability for these 
				  functions (outside of inequalities)
				- added support for infinite piecewises functions 
				  (Floor/Ceiling/Round/IntegerPart/FractionalPart/Quotient/Mod) over 
				  bounded domains
				- generalized piecewise transformations to nested and flat normal form
				- added support for debugging and measuring the performance of this 
				  code: DebugIntegrate, DebugNIntegrate
				- added additional notation for multiple integrals (Region[...])
				- robustness enhancements, i.e. trap more messages
				- sign change of integrals when regions given in nests are reversed
				- added support for nested infinite piecewise functions in inequalities 
				  and generally. 
				- added support for inequalities containing approximate numbers 
				- added message trapping for cleaner diagnostics
 *)
 
(* :Keywords: 
	Multiple Integration, 
	Region Integration, 
	Polynomial Inequalities, 
	Semi-Algebraic Sets,
	Piecewise Functions,
	Piecewise Algebraic Functions
 *)
 
(* :Sources: Ideas and implementation by Roger Germundsson *)

(* :Warning: *)

(* :Mathematica Version: 4.1 *)

(* :Copyright: Copyright 1998-2003, Wolfram Research, Inc.*)

(* :Limitation:
	Breaking down sets of inequalities relies on 
	  CylindricalDecomposition and as such can only deal with 
	  polynomial and algebraic inequalities. 
	Furthermore this stresses the capability of integrating algebraic and 
	  transcendental algebraic functions. 
 *)

BeginPackage[ "Calculus`Integration`" ];

Boole::usage =
"Boole[logic] is the characteristic function for a logical statement
containing equalities and inequalities.
Boole[True] == 1 and Boole[False] == 0";

Region::usage =
"Region[logic, vars] describes a region over which an integration is
performed. For instance, Integrate[expr, Region[ineqs, {x,y}]] is
equivalent to Integrate[expr Boole[ineqs], {x}, {y}]."

Integrate::region = "The region defined by `1` could not be broken down into cylinders.";

NIntegrate::region = "The region defined by `1` could not be broken down into cylinders.";



$MaxPiecewiseCases::infinite = "The range of values of `1` over the region `2` is infinite and \
and cannot be broken down into a finite piecewise function."
 
$MaxPiecewiseCases::large = "The range of values of `1` over the region `2` is `3` which is greater then \
$MaxPiecewiseCases = `4`. Increasing the value of $MaxPiecewiseCases may solve the problem."

$MaxPiecewiseCases::usage = "$MaxPiecewiseCases is the maximum number of piecewise cases allowed \
to be generated from either of Floor, Ceiling, Round, IntegerPart, FractionalPart, Mod and Quotient \
in functions like Integrate."

Begin[ "`Private`" ];

DebugIntegrate::usage = "DebugIntegrate[ expr, limits, opts ] performs the region and piecewise \
breakdown stage only and returns a sum of held interated integrals."

DebugNIntegrate::usage = "DebugNIntegrate[ expr, limits, opts ] performs the region and piecewise \
breakdown stage only and returns a sum of held interated numerical integrals."

Unprotect[ Integrate, NIntegrate, Boole, Region ]; 

(* Boole[ logic ] *)
Boole[ True ] = 1;
Boole[ False ] = 0;

(* ---------------- Symbolic Region Integration -------------------------*)

(* DebugIntegrate[ expr, ilimits, opts ]
 *
 *	Will perform the region breakdown stage only and the emit the resulting 
 *	iterated integrals as a sum of held iterated multiple integrals: 
 *	
 *		HoldForm[ Integrate[ expr, ilimits, opts ] ]
 *
 *	This provides a mechanism for debugging the work done in this package
 *	separate from work done the regular integration code. It also provides 
 *	for timing of this stage of process independent of the remaining integration
 *	stages. 
 *
 * $DebugIntegrate
 *	
 *	By setting this to True you can perform a whole session where the region
 *	breakdown only is performed in Integrate for appropriate integrals.
 *)	

$DebugIntegrate = False;

DebugIntegrate[ expr_, ilims__, opts___?OptionQ ] :=
	Block[ {$DebugIntegrate=True},
		Integrate[expr, ilims, opts ]
	];

integrate[ expr_, ilimits__, opts___?OptionQ ] := 
	HoldForm[ Integrate[ expr, ilimits, opts ] ];


(* MultipleIntegrate[ expr, ineqs, ilimits ]
 * Input: 
 *	expr	-- Expression to be integrated
 *	ineqs	-- Inequalities describing the region to be integrated over. 
 *	ilimits	-- Bounding box for integration. 
 * Output: 
 *	Symbolic integration of expr of the the region described by ineqs. 
 *)
 
(* Simple cases *)
MultipleIntegrate[ 0, ineqs_, vars_List, opts___?OptionQ ] := 0
MultipleIntegrate[ expr_, False, vars_List, opts___?OptionQ ] := 0
MultipleIntegrate[ expr_, True, vars_List, opts___?OptionQ ] :=
	If[ $DebugIntegrate === True,
		integrate[ expr, Sequence @@ ({#,-Infinity, Infinity}& /@ vars), opts ],
		Integrate[ expr, Sequence @@ ({#,-Infinity, Infinity}& /@ vars), opts ]
	];
	
(* General case *)
MultipleIntegrate[ expr_, ineqs_, vars_List, opts___?OptionQ ] :=
	Module[{nests, ints, ilim, cad},
			Check[ 
				Block[ {$CADMsgTrap=True}, cad = InequalityDecompose[ ineqs, vars ]; ], 
				(
				Message[ Integrate::region, ineqs ];
				Throw[ $Failed ]
				),
				CylindricalDecomposition::nrtpi,
				Experimental`GenericCylindricalAlgebraicDecomposition::nrtpi
			];
	    	nests 	= ToNestedLists[ cad, vars ];
			If[ $DebugIntegrate === True,
				ints 	= Apply[ integrate[ expr, ##, opts ]&, nests, {1} ],
				ints 	= Apply[ Integrate[ expr, ##, opts ]&, nests, {1} ]
			];
	    	If[ FreeQ[ ints, _Integrate ], Simplify[ Apply[ Plus, ints ] ], Apply[ Plus, ints ] ]
    ]

(* CheckILimits[ ... ]
	- Checks that it is the correct syntax (otherwise generates message).
	- Turns limits around when and adds a sign change when the upper 
	  limit is less then the lower one
 *)
CheckILimits[ {x_} ] := {1, {x, -Infinity, Infinity}};
CheckILimits[ {x_, l_, u_} ] := {1, {x, l, u}};
CheckILimits[ {x_, l_, u_} ] := {-1, {x, u, l}} /; l > u;
CheckILimits[ e_ ] := 
	(
	Message[ Integrate::ilim, e ];
	{1, e}
	);

(* CondiditionalIntegrate[ pwexpr, ilimits, opts ]
 *	This function unravels the piecewise structure in expr and generate a sequence
 *	of MultipleIntegrate[ expr, region, vars, opts ] calls, i.e. integration over 
 *	regions. 
 *
 *	There can easily be a piecewise expression blowup in the the number of cases 
 *	and in the complexity of the resulting conditions or regions. 
 *
 *	There are several sources of case blowup: 
 *	1. Removing dependence on program flow or evaluation order in Which expressions. 
 *		Solution: The only case where the evaluation order is a problem is with 
 *			user input of Which expressions, e.g. Integrate[ Which[0<x<2, 1, 1<x<3, 2], {x}]
 *			should evaluate to 4 and not 6. We remove the dependence only for user input
 *			all internal transformations are evaluation order safe. 
 *	2. Construction of many cases that evaluate to 0
 *		Solution: These things come naturally out of constructions such as UnitStep, Boole etc, 
 *			but we can easily (and cheaply) remove them through whichSimplify.
 *)
$FinitePiecewise = True;
$InfinitePiecewise = True;

ConditionalIntegrate[ expr_, ilimits_List, opts___?OptionQ ] :=
	Module[ {sign, ilims, nexpr, ineqs, exprs, conds, vars, nexprs, nineqs, pwexpr},
		Check[ 
			{sign, ilims} = Transpose[ CheckILimits /@ ilimits ], 
			Return[ $Failed ] 
		];
		
		vars  = Map[ First, ilims ];
		nexpr = whichSimplify[
			ExtractConditionals[ (Times @@ sign) RemoveProgramFlow[ expr ] Boole[ FromNestedLists[ {ilims} ] ] ]
			];
		{ineqs, exprs} = pwUnpack[ nexpr ];

		(* Infinite Piecewise: Floor, Ceiling, Round, IntegerPart, FractionalPart, Mod, Quotient *)
		If[ $InfinitePiecewise === True,
			pwexpr = InfiniteConditionalFunctions[ nexpr, vars ];
			If[ FreeQ[ pwexpr, $Failed ] && FreeQ[ pwexpr, $infinitePiecewisePattern ],
				nexpr = pwexpr;
			];
			{ineqs, exprs} = pwUnpack[ nexpr ];
		];
		
		(* Finite Piecewise: Abs, Min, Max, UnitStep, Sign, Boole *)
		If[ $FinitePiecewise === True, 
			nexpr = FiniteConditionalFunctions[ nexpr ];
			{ineqs, exprs} = pwUnpack[ nexpr ];
		];
		
		Catch[ 
		 Apply[ 
			Plus, 
			MapThread[ MultipleIntegrate[ #2, #1, vars, opts ]&, {ineqs, exprs} ]
		 ]
		]
	]
	
DefaultILimits[ {x_} ] := {x, -Infinity, Infinity};
DefaultILimits[ any_ ] := any;	(* Fall through to regular Integrate argument checking *)
	
Integrate[ expr_, ilimits___List, opts___?OptionQ ] :=
	Integrate[ expr, Sequence @@ (DefaultILimits /@ {ilimits}), opts ] /; Or @@ Map[ Length[#]==1&, {ilimits} ]

Integrate[ expr_, ilimits___List, opts___?OptionQ ] :=
	cint /; !FreeQ[ expr, _UnitStep | _Boole | _If | _Which | _Max | _Min ] && 
			FreeQ[ expr, _DiracDelta ] &&
			FreeQ[ cint = ConditionalIntegrate[ expr, {ilimits}, opts ], $Failed ]
		
(* Note: This now hits recursion limit, when we fail to integrate e.g. Floor
!FreeQ[ expr, _Floor | _Ceiling | _Round | _Mod | _Quotient | _IntegerPart | _FractionalPart ]*)

Integrate[ expr_, Region[ ineqs_, vars_List ], opts___?OptionQ ] :=
	Integrate[ expr Boole[ ineqs ], Sequence @@ ({#, -Infinity, Infinity}& /@ vars), opts ];
	
(* PIntegrate[ expr, limits, opts ]
 *
 * 	Implements experimental support for multiple integrals over regions containing 
 *	parameters. 
 *
 *	This is currently not generally supported, mainly because we need to have more 
 *	general support for piecewise functions throughout Mathematica. 
 *
 * 	Additional issues: Assumptions input, additional piecewise functions, parameters 
 *		in integration limits. 
 *)
PIntegrate[ expr_, ilimits___List, opts___?OptionQ ] :=
	Module[ {nexpr, ineqs, exprs, vars, pars},
		nexpr = ExtractConditionals[ expr ]; 
		{ineqs, exprs} = Transpose[ Partition[ Apply[ List, nexpr ], 2 ] ];
		vars = Map[ First, {ilimits} ];
		pars = Complement[ Apply[ Union, Map[ variables, ineqs ]], vars ]; 
		(* Simplified version ... ok for one Boole[ .. ] expr. *)
		PMultipleIntegrate[ exprs[[1]], ineqs[[1]], pars, {ilimits}, opts ]
	]
	
(* Simple cases *)
PMultipleIntegrate[ 0, ineqs_, pars_, ilimits_List, opts___?OptionQ ] := 0
PMultipleIntegrate[ expr_, False, pars_, ilimits_List, opts___?OptionQ ] := 0
PMultipleIntegrate[ expr_, True, pars_, ilimits_List, opts___?OptionQ ] :=
	Integrate[ expr, Sequence @@ ilimits, opts ]

(* General case *)
PMultipleIntegrate[ expr_, ineqs_, pars_, ilimits_List, opts___?OptionQ ] :=
	Module[{nests, ints, pineqs, vars, ilim, cad, conds},
			vars	= Map[ First, ilimits ];
			ilim	= FromNestedLists[ {ilimits} ];
			pineqs 	= ineqs && ilim;
			Check[ 
				cad = InequalityDecompose[ pineqs, Join[ pars, vars ] ], 
				(
				Message[ Integrate::region, pineqs ];
				Return[ $Failed ]
				)
			];
	    	nests 	= ToNestedLists[ cad, Join[ pars, vars ] ];
			conds	= Union[ Take[ nests, All, Length[ pars ] ] ];
			tab 	= Table[
				cnests = 
					Take[
						Select[ nests, (Take[#, Length[pars]] === conds[[i]])& ], 
						All, -Length[vars]
					];
				cond = FromNestedLists[ {conds[[i]]} ];
				Rule[
					cond,
					Apply[ Plus, Apply[ Integrate[ expr, ##, Assumptions -> cond ]&, cnests, {1} ] ]
				], 
				{i, Length[ conds ]}
			];
			Apply[ Which, Flatten[ Apply[ List, tab, {1} ] ] ]	
    ]

(* ----------------- Numerical Region Integration -------------------*)

(* DebugNIntegrate[ expr, ilimits, opts ]
 *
 *	Will perform the region breakdown only stage and the emit the resulting 
 *	iterated integrals as a sum of held iterated multiple integrals: 
 *	
 *		HoldForm[ NIntegrate[ expr, ilimits, opts ] ]
 *
 *	This provides a mechanism for debugging the work done in this package
 *	separate from work done the regular integration code. It also provides 
 *	for timing of this stage of process independent of the remaining integration
 *	stages. 
 *
 *	$DebugNIntegrate
 *
 *	Will perform the region breakdown only stage for all appropriate integrals.
 *)	
 
$DebugNIntegrate = False;

DebugNIntegrate[ expr_, ilims__, opts___?OptionQ ] :=
	Block[ {$DebugNIntegrate=True},
		NIntegrate[expr, ilims, opts ]
	];

nintegrate[ expr_, ilimits__, opts___?OptionQ ] := 
	HoldForm[ NIntegrate[ expr, ilimits, opts ] ];
	
(* Simple cases *)
MultipleNIntegrate[ 0, ineqs_, vars_List, opts___?OptionQ ] := 0.
MultipleNIntegrate[ expr_, False, vars_List, opts___?OptionQ ] := 0.
MultipleNIntegrate[ expr_, True, vars_List, opts___?OptionQ ] :=
	If[ $DebugNIntegrate === True,
			nintegrate[ expr, Sequence @@ ({#, -Infinity, Infinity}& /@ vars), opts ],
			NIntegrate[ expr, Sequence @@ ({#, -Infinity, Infinity}& /@ vars), opts ]
	];
	
(* General case *)
MultipleNIntegrate[ expr_, ineqs_, vars_List, opts___?OptionQ ] :=
	Module[{nests, ints, cad, wprec, nineqs},
			wprec = WorkingPrecision /. {opts} /. Options[ NIntegrate ];
			nineqs = If[ wprec <= $MachinePrecision,  	N[ ineqs, $MachinePrecision + 1 ], 
														N[ ineqs, wprec ] ]; (* Performance optimization *)
			Check[ 
				Block[ {$CADMsgTrap=True}, cad = InequalityDecompose[ nineqs, vars ]; ], 
				(
				Message[ NIntegrate::region, ineqs ];
				Return[ $Failed ]
				),
				CylindricalDecomposition::nrtpi,
				Experimental`GenericCylindricalAlgebraicDecomposition::nrtpi
			];
	    	nests 	= ToNestedLists[ cad, vars ];
			If[ $DebugNIntegrate === True,
				ints 	= Apply[ nintegrate[ expr, ##, opts ]&, nests, {1} ],
				ints 	= Apply[ NIntegrate[ expr, ##, opts ]&, nests, {1} ]
			];
	    	Apply[ Plus, ints ]
    ]

(* Note: There are several ways in which this code can be installed 
	into regular NIntegrate. For instance as can be seen below we 
	can install the code in a condition and when it fails we proceed 
	with regular NIntegrate evaluation. However as of right now '98
	this code is not equipped to deal with discontinuities like this 
	and basically hangs or take a very long time. Right now we return 
	$Failed.
	See ConditionalNIntegrate etc. 
   Note: Possibly should Rationalize inequalities as well at least until 
   CAD is able to deal with floating point. 
 *)
 
(* CheckNILimits[ ... ]
	- Checks that it is the correct syntax (otherwise generates message).
	- Turns limits around when and adds a sign change when the upper 
	  limit is less then the lower one
 *)
CheckNILimits[ {x_} ] := {1, {x, -Infinity, Infinity}};
CheckNILimits[ {x_, l_, int___, u_} ] := {1, {x, l, u}};
CheckNILimits[ {x_, l_, int___, u_} ] := {-1, {x, u, l}} /; l > u;
CheckNILimits[ e_ ] := 
	(
	Message[ NIntegrate::vars, e ];
	{1, e}
	);

ConditionalNIntegrate[ expr_, ilimits_List, opts___?OptionQ ] :=
	Module[ {nexpr, ineqs, exprs},
		Check[ 
			{sign, ilims} = Transpose[ CheckNILimits /@ ilimits ], 
			Return[ $Failed ] 
		];
		
		vars  = Map[ First, ilims ];
		nexpr = whichSimplify[ 
			ExtractConditionals[ (Times @@ sign) RemoveProgramFlow[ expr ] Boole[ FromNestedLists[ {ilims} ]] ]
			];
		{ineqs, exprs} = Transpose[ Partition[ Apply[ List, nexpr ], 2 ] ];
		
		Apply[ 
			Plus, 
			MapThread[ MultipleNIntegrate[ #2, #1, vars, opts ]&, {ineqs, exprs} ]
		]
	]
	
NIntegrate[ expr_, ilimits___List, opts___?OptionQ ] :=
	NIntegrate[ expr, Evaluate[ Sequence @@ (DefaultILimits /@ {ilimits}) ], opts ] /; Or @@ Map[ Length[#]==1&, {ilimits} ]

NIntegrate[ expr_, ilimits___List, opts___?OptionQ ] :=
	cint /;	!FreeQ[ expr, _UnitStep | _Boole | _If | _Which ] &&
			FreeQ[ cint = ConditionalNIntegrate[expr, {ilimits}, opts], $Failed ]
			
NIntegrate[ expr_, Region[ ineqs_, vars_List ], opts___?OptionQ ] :=
	Apply[ NIntegrate[ expr Boole[ ineqs ], ##, opts ]&, {#,-Infinity, Infinity}& /@ vars ];
	
(*----------- Inequality Decomposition and Conversion --------------------*)

(* InequalityDecompose[ ineq, vars ]
 * 
 * Will decompose a set of inequalities. It will return the 
 * full dimensional (of dimension == Length[vars]) only. 
 * This indirection will be useful in the future when different 
 * functionality will be used for the decomposition step.
 *)

InequalityDecompose[ ineqs_, vars_ ] :=
	Module[ {nineqs, cad, prec = Precision[ ineqs ]},
		If[ TrueQ[ prec < Infinity ], nineqs = Rationalize[ ineqs, 0 ], nineqs = ineqs ];
		If[ AlgebraicQ[ ineqs, vars ], 
			cad = SelectCylinders[ 
				CylindricalDecomposition[ nineqs, vars ], vars, Length[vars]
			],
			cad = First[ Experimental`GenericCylindricalAlgebraicDecomposition[ nineqs, vars ] ]
		];
		If[ TrueQ[ prec < Infinity ], N[ cad, prec ], cad ]
	];

AlgebraicQ[ expr_, vars_ ] :=
	!FreeQ[ expr, _Sqrt|Power[_,_Rational]|_Root ];
	
Unprotect[ Message ];

$CADMsgTrap = False;

msg : Message[ CylindricalDecomposition::nrtpi, args___ ] /; $CADMsgTrap :=
	Block[ {$CADMsgTrap=False, $Messages={}}, msg ];

msg : Message[ Experimental`GenericCylindricalAlgebraicDecomposition::nrtpi, args___ ] /; $CADMsgTrap :=
	Block[ {$CADMsgTrap=False, $Messages={}}, msg ];
	
Protect[ Message ];

(* 
 * The following functions implement a set of utilities for dealing with nests 
 * and inequalities in cylindric (or nested) form. 
 *
 * Main functions:
 * 
 *	ToNestedLists[ ineqs, vars ] ==> nests
 *	FromNestedLists[ nests ] ==> ineqs
 *
 * Inequalities in nested or cylindric form: 
 *
 *	a1 < x1 < b1 && a2[x1] < x2 < b2[x1] && ... && an[x1,..,xn-1] < xn < bn[x1,..,xn-1] ||
 *	c1 < x1 < d1 && c2[x1] < x2 < d2[x1] && ... && cn[x1,..,xn-1] < xn < dn[x1,..,xn-1] ||
 *	...
 *
 * where the inequality < can be either of < or <= and similarly > either of > and >=.
 *
 * An equivalent representation in terms of nestings (as used in Integrate, NIntegrate etc) is 
 * of the form: 
 * {
 *	{ {x1, a1, b1}, {x2, a2[x1], b2[x1]}, ..., {xn, an[x1,..,xn-1], bn[x1,..,xn-1]} },
 *	{ {x1, c1, d1}, {x2, c2[x1], d2[x1]}, ..., {xn, cn[x1,..,xn-1], dn[x1,..,xn-1]} }, 
 *	...
 * }
 *
 * ToNestedLists and FromNestedLists convert to and from these representations. 
 *
 * Additional notes: 
 *
 *	o The nesting representation which is essentially a generalization of an Interval, 
 *		cannot distinguish between open and closed intervals or equivalently between 
 *		strict and non-strict inequalities. By adding a function such as DirectedZero[+1] etc
 *		the nesting representation can distinguish betwen open and closed intervals. 
 *
 *	o Equalities show up in nestings with lower and upper bound equal. Unequal cannot 
 *		be represented. 
 *
 *	o CylindricalDecomposition generates a set of inequalities of appropriate 
 *		cylindric or nested form. 
 *
 * Additional functions:
 *
 *	SelectCylinders[ ineqs, vars, dim ] ==> ineqs
 *		Selects the inequalities defining a set of dimensions dim.
 *
 *	The following define certain normal forms for logical combinations:
 *	
 *	ToClauseList[ ineq1 || ... || ineqn ] ==> {ineq1, .., ineqn}
 *	FromClauseList[ {ineq1, .., ineqn} ] ==> ineq1 || ... || ineqn
 *
 *	ToLiteralLists[ a1 && ... && an || b1 && ... && bm || ... ] ==>
 *		{{a1,...,an}, {b1,...,bm}, ... }
 *	FromLiteralLists[ {{a1,...,an}, {b1,...,bm}, ... } ] ==> 
 *		a1 && ... && an || b1 && ... && bm || ...
 *
 * 
 *)

(* 
 * ToClauseList[ logic ]
 * FromClauseList[ list ]
 *)
ToClauseList[ ineqs_ ] :=
  toClauseList[ LogicalExpand[ ineqs ] ];

ToClauseList[ ineqs_List ] :=
  Flatten[ Map[ ToClauseList, ineqs ] ];

toClauseList[ ineq_Or ] := Apply[ List, ineq ];

toClauseList[ ineq_ ] := {ineq};

FromClauseList[ list_List ] :=
	Apply[ Or, list ]

(* 
 * ToLiteralLists[ logic ]
 * FromLiteralLists[ list ]
 *)
ToLiteralLists[ ineqs_ ] :=
  Map[ toLiteralList, ToClauseList[ ineqs ] ];

ToLiteralLists[ ineq_List ] :=
  ToLiteralLists[ FromLiteralLists[ ineq ] ];

toLiteralList[ ineq_And ] := Apply[ List, ineq ];

toLiteralList[ ineq_ ] := {ineq};

FromLiteralLists[ list_List ] :=
	Apply[ Or, Apply[ And, list, {1} ] ];
 
(* ToNestedLists[ ineqs, vars ]
 * Note: This works only for full dimensional components. 
 * Basic parsing of inequalities generated by CAD. 
 *
 * 	1. [ToLiteralLists]
 *		Compute a normal form which is a list of list corresponding to 
 *		Or:s of And:s. To separate the various cylinders.
 *	2. [toNestedComponentList]
 *		Pick out the various components for each variable, i.e. of the form: 
 *		{ {x1, ineq[x1]}, {x2, ineq[x1, x2]}, ..., {xn, ineq[x1,..,xn-1]}}
 *	3. [toBounds]
 *		Get the actual bounds for the main variable in the list of ineqs. 
 *	3.1 [findComponentBounds]
 *		Parse the list of inequalities using findBound and combine the result. 
 *	3.1.1 [findBound]
 *		Parse elementary inequalities.
 *	3.2 [toStandardBounds]
 *		Compute standard representation, i.e. using Infinity when no bound is present.
 *)
ToNestedLists[ fcad_, vars_ ] := 
	Map[ 
		toNestedList[#, vars]&, 
		ToLiteralLists[ LogicalExpand[ fcad ] ]
	];

(* The full R^n *)
ToNestedLists[ True, vars_ ] :=
	List[
		Map[ {#, -Infinity, Infinity}&, vars ]
	];
	
(* The empty set in R^n -- perhaps it should return $Failed *)
ToNestedLists[ False, vars_ ] :=
	List[
		Map[ {#, 0, 0}&, vars ]
	];

toNestedList[ cyl_, vars_ ] :=
	Map[ 
	  	toBounds,
		toNestedComponentList[ cyl, vars ]
	];

toNestedComponentList[ cyl_List, vars_List ] :=
  toNestedComponentList[ cyl, vars, {}, {} ];
  
(* Iterative function that finds bounds for one variable at the time *)
toNestedComponentList[ rcomps_List,{xf_, xr___}, dcomps_List, dvars_List ] :=
  Module[ {newc},
    newc = Select[ rcomps, FreeQ[#, Alternatives[xr]]& ];
    toNestedComponentList[
      Complement[ rcomps, newc ], 
      {xr}, 
      Append[ dcomps, { xf, newc } ], 
      Append[ dvars, xf ]
      ]
    ];

toNestedComponentList[ rcomps_List, {}, dcomps_List, dvars_List ] := dcomps;

toBounds[ {x_, {}} ] := 
	toStandardBounds[ {x, {}, {}} ];
toBounds[ {x_, elineqs_List} ] :=
  	toStandardBounds[ findComponentBounds[ {x, elineqs} ] ];

(* toStandardBounds[ {x, {lower}, {upper}} ] --> {x, lower, upper} *)
toStandardBounds[ {x_, {}, {}} ] 	:= {x, -Infinity, Infinity};
toStandardBounds[ {x_, {}, hi_} ] 	:= toStandardBounds[{x, {-Infinity}, hi}];
toStandardBounds[ {x_, lo_, {}} ] 	:= toStandardBounds[{x, lo, {Infinity}}];
toStandardBounds[ {x_, {lo_},{ hi_ }} ] := {x, lo, hi};
toStandardBounds[ b:{x_, lo_, hi_} ] :=
  (Message[ cylinder::bounds, x, b ];b);

(* findComponentBounds[ {x, {ineq1, ..., ineqn}} ] --> {x, {lower}, {upper}} *)
findComponentBounds[ {x_, elineqs_List} ] :=
  	Prepend[
    	Apply[ Join, Transpose[ Map[ findBound[#, x]&, elineqs ] ], {1} ], 
		x 
	];

(* findBound[ x < expr, x ] --> {{}, {expr}} *)
findBound[ (Less|LessEqual)[x_, expr_], x_ ] := {{}, {expr}};
findBound[ (Less|LessEqual)[expr_, x_], x_ ] := {{expr}, {}};
findBound[ (Greater|GreaterEqual)[x_, expr_], x_ ] := {{expr}, {}};
findBound[ (Greater|GreaterEqual)[expr_, x_], x_ ] := {{}, {expr}};

(* FromNestedLists[ ineqs, vars ]
 *
 *)
FromNestedLists[ nests_List ] :=
  Apply[ Or, Map[ Apply[ And, Map[ boundToInequality, # ] ]&, nests ] ];

boundToInequality[ {x_} ] 						:= True;
boundToInequality[ {x_, -Infinity, Infinity} ] 	:= True;
boundToInequality[ {x_, -Infinity, ux_} ] 		:= x<= ux;
boundToInequality[ {x_, lx_, Infinity} ] 		:= x>=lx;
boundToInequality[ {x_, lx_, ux_} ] 			:= lx <= x <= ux;

(* SetDimension[ cad ]
 *
 *)
SetDimension[ cad_, vars_ ] := 
  Max[ 
    Map[
      cylinderDimension[#,vars]&,
      ToClauseList[ LogicalExpand[ cad ] ]
      ]
    ];

cylinderDimension[ cyl_, vars_ ] :=
  Length[ vars ] - Count[ cyl, _Equal, {0, Infinity} ];

SelectCylinders[ cad_, vars_, dim_ ] :=
	Apply[ Or, 
		Select[ 
			ToClauseList[LogicalExpand[cad]],
			(cylinderDimension[#,vars] == dim)&
		]
	];
	
SelectCylinders[ True, vars_, dim_ ] := True;
SelectCylinders[ False, vars_, dim_ ] := False;

(*--------------- Parameters in Region Descriptions ----------------------*)
	
(* variables[ ineqs ]
 * Input: 	Logical combination of inequalities and equalities
 * Output:	Set of variables under the constructors below
 *
 * Note: 	These sets are described by a number of constructors
 *			from which we can isolate the notion of variables we have to use. 
 *
 *	poly	::=	Numbers | Times[ poly, ..., poly ] | Plus[ poly, ..., poly ] | Power[ poly, Integer (>= 0)]
 *	alge	::= poly | Sqrt[ alge ] | Power[ alge, Rational ] | Root[ poly, k ]
 *  algext	::= alge | Abs[ algext ] | Min[ algext, ..., algext ] | Max[ algext, ..., algext ]
 *	eq		::= Equal[ algext, ..., algext ] | Unequal[ algext, ..., algext ]
 *	ineq	::= eq | Less[ algext, algext ] | LessEqual[ algext, algext ] |
 *				Greater[ algext, algext ] | GreaterEqual[ algext, algext ] |
 *				Inequality[ algext, comp, algext, comp, ..., comp, algext ]
 *	semi	::= Xor[ ineq, .., ineq ] | And[ ineq, .., ineq ] |
 *				Or[ ineq, .., ineq ] | Implies[ ineq, ineq ] | Not[ ineq ]
 *
 *	Numbers are not variables, but everything else is.
 *)
 
variables[(And | Or | Xor | Implies | Not)[e__] ] :=
  Apply[ Union, Map[ variables, {e}] ]

variables[(Equal | Unequal | Less | LessEqual | Greater | GreaterEqual)[ e__ ] ] :=
  Apply[ Union, Map[ variables, {e} ]]

variables[ e_Inequality ] :=
  Module[ {ee = Apply[List, e]}, 
    Apply[ Union, Map[ variables, Take[ ee, {1, Length[ee], 2}]]]
  ]

variables[ (Plus | Times | Sqrt)[e__] ] :=
  Apply[ Union, Map[ variables, {e} ]]

variables[ Root[ poly_, k_ ] ] :=
  Module[ {x}, Complement[ Variables[ poly[x] ], {x} ] ]

variables[ Power[ e_, _Rational | _Integer ] ] :=
  variables[ e ]
  
variables[ (Max | Min | Abs)[ e__ ] ] :=
  Apply[ Union, Map[ variables, {e} ]]

variables[ e_?NumericQ ] := {}

variables[ e_ ] := {e}

(*----------------- General Transformations for Piecewise Expressions ----------------- *)


ExtractNestedConditionals::usage = "ExtractNestedConditionals[ expr ] will extract all inline conditional \
expressions (UnitStep, Boole, If, Which) into a form where the conditions are in the outer \
most form and including only (Which, If)."

ExtractConditionals::usage = "ExtractConditionals[ expr ] will extract all inline \
conditional expression (UnitStep, Boole, If, Which) into a form where the the conditions are \
in the outer most form and only one level deep and including only (Which)."

CaseForm::usage = "CaseForm[ e ] display typeset form of nested expression involving conditional \
expressions. The different cases are expressed in a flattened form."

(* Additional functions more intended for development *)

removeUnitStep::usage = "removeUnitStep[ expr ] will rewrite UnitStep expressions in terms of \
Boole expressions."

removeBoole::usage = "removeBoole[ expr ] will rewrite Boole expressions in terms of If \
expressions."

removeIf::usage = "removeIf[ expr ] will rewrite If expressions in terms of Which expressions."

flattenWhich::usage = "flattenWhich[ expr ] assumes that expr is a set of nested Which statements \
with all Which expression occurring in the top most level into a form where only one level of Which \
occurs."

removeProgramFlow::usage = "removeProgramFlow[ expr ] rewrites Which expressions in a form where \
they are independent of evaluation order (assuming no side-effects)."

whichForm::usage = "whichForm[ expr ] produces a nice typeset form of nested Which statements."

fromConditional::usage = "fromConditional[ expr ] turns If->if and Which->which in order to \
be able to manipulate these expressions without special evaluation semantics attached."

toConditional::usage = "toConditional[ expr ] turns if->If and which->Which in order to \
return regular Mathematica conditional expressions."

which::usage = "which[ c1, e1, c2, e2, ... ] substitute for Which without any special evaluation \
rules attached to it."

if::usage = "if[ c, e1, e2 ] substitute for If without any special evaluation rules \ 
attached to it."


(* Need the right boundary case for this, i.e. 
 *		extract[ expr, {} ] ==> expr
 *		replacePart[ expr, sexpr, {} ] ==> sexpr
 *)
extract[ expr_, pos_ ] :=
  If[ pos === {}, expr, Extract[ expr, pos ] ];
  
replacePart[ expr_, sexpr_, pos_ ] :=
  If[ pos === {}, sexpr, ReplacePart[expr, sexpr, pos]];
  
(* 
 * pwUnpack[ which[ c1, e1, c2, e2, ... ] ] ==> {{c1,c2,..}, {e1, e2, ..}}
 * pwPack[ {{c1,c2,..}, {e1, e2, ..}} ] ==> which[ c1, e1, c2, e2, .. ]
 *
 * These are common idioms when manipulating piecewise expressions. 
 *)
pwUnpack[ expr_which ] := Transpose[Partition[Apply[List, expr], 2]];

(* Which[ False, anything ] and If[ False, anything ] autosimiplifies to Null *)
pwUnpack[ Null ] := {{False},{0}};

(* We need to be prepared that we are not unraveling all pw functions in one go. *)
pwUnpack[ expr_ ] /; FreeQ[ expr, _which ] := {{True}, {expr}};

pwPack[ {conds_List, exprs_List} ] := 
  Apply[ which, Flatten[ Transpose[ {conds, exprs} ] ] ];
  
(* To avoid funny rules attached to If and Which, I need to protect which/if. *)
fromConditional[ expr_ ] :=
	removeIf @ removeBoole @ removeUnitStep @ ReplaceAll[ expr, {Which->which, If->if} ];

(* This doesn't work, we can stick to which for now ... 
toConditional[ expr_ ] :=
	MapAll[ Replace[ #, {which->Which, if->If}]&, expr, Heads -> True ];
 *)

(* Simple syntactic rewrites. Note: Remove the context when Boole moves out... *)
removeUnitStep[ expr_ ] := 
	ReplaceRepeated[
		expr,
		UnitStep[ e__ ] :> Boole[ Apply[ And, Map[ (# >= 0) &, {e}]] ]
	];

removeBoole[ expr_ ] :=
	ReplaceRepeated[
		expr, Boole[ e_ ] :> If[ e, 1, 0 ]
	];

(* May need some temporary symbol here as Which have special evaluation rules. 
 * Note: The special forms: If[ c, e ] effectively is translated to If[ c, e, 0 ]
 *)
removeIf[ expr_ ] :=
  ReplaceRepeated[ 
  	expr, 
    {If[ c_, e1_, e2_ ] :> which[c, e1, Not[c], e2], 
     If[ c_, e1_ ] :> which[ c, e1, Not[c], 0 ], 
	 if[ c_, e1_, e2_ ] :> which[ c, e1, Not[c], e2 ], 
	 if[ c_, e1_ ] :> which[ c, e1, Not[c], 0 ]} 
    ];

normalizeIf[ expr_ ] :=
	ReplaceRepeated[ 
	  	expr, 
	    {
	     If[ c_, e1_ ] :> If[ c, e1, 0 ], 
		 if[ c_, e1_ ] :> if[ c, e1, 0 ]
		} 
    ];
	
(* PiecewiseNormalize[ expr ]
 *
 * Rewrites the following piecewise functions:
 *
 *	Sign, UnitStep, Boole, If
 *
 * in terms of Which.
 *
 * Note: We don't want to rewrite Abs, Min, Max as these are handled 
 *	more efficiently in inequality solving directly. 
 *)
PiecewiseNormalize[ expr_ ] :=
	ReplaceRepeated[ expr,
		{
		UnitStep[ e__ ] 	:> if[ Apply[ And, Map[ (# >= 0) &, {e}]], 1, 0 ],
		Sign[ e_ ]			:> which[ e>0, 1, e==0, 0, e<0, -1],
		Boole[ c_ ] 		:> if[ c, 1, 0 ],
		If[ c_, e1_, e2_ ]	:> which[ c, e1, !c, e2 ],
		If[ c_, e1_ ]		:> which[ c, e1, !c, 0 ],
		if[ c_, e1_, e2_ ]	:> which[ c, e1, !c, e2 ], 
		if[ c_, e1_ ] 		:> which[ c, e1, !c, 0 ]
		}
	] /. which -> Which;

flattenWhich[ expr_ ] :=
	Module[ {cc, ee, iexpr}, 
  		iexpr = fromConditional[ expr ];    
		ReplaceRepeated[ 
      		expr, 
      		which[ first___, c_, e_which, rest___ ] :>
        	({cc, ee} = Transpose[ Partition[Apply[List, e], 2]];
          	which[ 
            	first, 
            	Sequence @@ Flatten[Transpose[ {Map[ # && c &, cc], ee}]], 
            	rest
            ])
      	]
	]

(* removeProgramFlow[ expr ]
 * Input:
 *	expr	-- A nested expression consisting of if and which expressions. 
 * Output: 
 *	An equivalent to expr, but where evaluation order no longer matters, 
 *	assuming that there are no side-effects. 
 *	This effectively performs the operation: 
 *	Which[ c1, e1, c2, e2, .., cn, en ] -->
 *		Which[ c1, e1, !c1&&c2, e2, !c1&&!c2&&c3, e3, ... ]
 *)
removeProgramFlow[ expr_ ] :=
  	Module[ {nexpr, pos, ewhich, c, e},
		nexpr = expr /. Which -> which;
      	pos = Last[ Position[ nexpr, _which ] ];
      	ewhich = extract[ nexpr, pos ];
      	{c, e} = Transpose[ Partition[Apply[ List, ewhich], 2] ];
      	cnew = 
        	Table[
          		Apply[And, Map[Not, Take[c, {1, k - 1}]]] && Part[c, k],
          		{k, Length[c]}
          	];
      	removeProgramFlow[
        	replacePart[ nexpr, 
          		Apply[ which1, Flatten[ Transpose[{cnew, e}]]], pos ]
        ]
      ] /; ! FreeQ[ expr, _which | _Which ]

removeProgramFlow[ expr_] :=
	expr /. which1 -> which;

RemoveProgramFlow[ expr_ ] :=
	removeProgramFlow[ expr ] /. which -> Which;


(* New Piecewise Transformation Functions
 * Basic Strategy:
 * 
 *	1. Use the local distributive law in a bottom-up fashion. 
 *
 *		If[ c, e1[ If[ d, f1, f2 ] ], e2 ] -->
 *			If[ c, If[ d, e1[ f1 ], e1[ f2 ] ], e2 ]
 *
 *		(see separate report for full details)
 *	2. Remove all piecewise nesting in the logical expressions. 
 *		a. Type inferencing gives locations of pure logical nesting.
 *		b. Logical transformation removes these piecewise expressions.
 *
 *		If[ If[ c, e1, e2 ], f1, f2 ] --> If[ c && e1 || !c && e2, f1, f2 ]
 *
 *	3. Convert to a flattened normal form of the resulting nested 
 *		normal form piecewise expression. 
 *
 * Notes: We should not go inside objects with held arguments, except of course 
 *	If and Which that this functionality is designed to transform. 
 *)

(* Produces a nested piecewise expression with no piecewise expressions in the logical 
	parts *)
	
(* For testing, set DenestLogicalPWForm = DenestLogicalPWForm1 or DenestLogicalPWForm2 *)

DenestLogicalPWForm = DenestLogicalPWForm1;

NestedPWForm[ pwexpr_ ] :=
    DenestLogicalPWForm @ nestedPWForm[ pwexpr ];
 
(* Find positions that haven't yet been transformed into nested form and 
	perform transformation *)

nestedPWForm[ pwexpr_ ] := 
  Module[{pos},
      pos = Last[Position[pwexpr, _If | _Which]];
      localPWDistribute[pwexpr, extract[pwexpr, pos], pos]
      ] /; ! FreeQ[pwexpr, _If | _Which]

nestedPWForm[ pwexpr_ ] := pwexpr;

(* Apply the distributive law for If[ ... ] *)

localPWDistribute[ pwexpr_, If[ c_, e1_, e2_], pos_ ] :=
    Map[
      nestedPWForm,
      if[ c, 
        replacePart[ pwexpr, e1, pos ],
        replacePart[ pwexpr, e2, pos ]
        ]
      ];

(* Apply the distributive law for Which[ ... ] *)

localPWDistribute[ pwexpr_, sexpr_Which, pos_ ] :=
    Map[ nestedPWForm, 
      which @@ 
        Table[
          If[OddQ[i],
            sexpr[[i]],
            replacePart[pwexpr, sexpr[[i]], pos]
            ],
          {i, Length[sexpr]}
          ]
      ];


DenestLogicalPWForm2[ e_ ] := 
	MapAll[ denestLogicalPWForm, e ];

(* Type inference logical values and do denesting transformation *)

denestLogicalPWForm[ if[ c_, e1_, e2_ ] ] :=
    if[ denestLogical //@ c, e1, e2 ];

denestLogicalPWForm[ which[ ce__ ] ] :=
    Module[ {c, e}, 
      {c, e} = Transpose[ Partition[ {ce}, 2] ];
      which @@ Flatten[ Transpose[ {denestLogical //@ c, e} ] ]
    ];

denestLogicalPWForm[ pw_ ] := pw;

(* Perform logical denesting transformation *)

denestLogical[ if[ c_, e1_, e2_ ] ] :=
  	Simplify //@ (c && e1 || ! c && e2) /; FreeQ[ {c, e1, e2}, _if | _which ];

denestLogical[ pw_which ] :=
    Module[ {c, e, flow}, 
        {c, e} = Transpose[ Partition[ List @@ pw, 2] ];
        flow = Table[ And @@ (Not /@ Take[ c, k - 1]), {k, Length[c]}];
        Simplify //@ (Or @@ Thread[ And[ flow, c, e ] ])
    ] /; FreeQ[ List @@ pw, _if | _which ];

denestLogical[ pw_ ] := pw;

(* A different implementation of logical denesting, it has slightly different 
   performance characteristic, but is otherwise more robust against held 
   arguments such as in Root. *)

DenestLogicalPWForm1[ e_ ] :=
	generalDenest[ e ] /. {if1->if, which1->which};



generalDenest[ e_ ] := e /; FreeQ[ e, _if|_which ];	(* Shortcut *)

generalDenest[ if[ c_, e1_, e2_ ] ] :=				(* Type branching *)
    if1[ logicalDenest[c], generalDenest[e1], generalDenest[e2] ];
	
generalDenest[ pw_which ] :=						(* Type branching *)
    which1 @@ Table[ If[ OddQ[i], logicalDenest[pw[[i]] ], generalDenest[ pw[[i]] ] ], {i, Length[pw]} ];

generalDenest[ e_?AtomQ ] := e;						(* General case *)

generalDenest[ h_[ e___ ] ] := 						(* General case *)
    h[ Sequence @@ Map[ generalDenest, {e} ] ];



logicalDenest[ e_ ] := e /; FreeQ[ e, _if | _which ]; (* Shortcut *)

logicalDenest[ if[ c_, e1_, e2_ ] ] := 				(* Denesting *)
	Module[ {cc, ee1, ee2},
		{cc, ee1, ee2} = Map[ logicalDenest, {c, e1, e2} ];
    	Simplify[ cc && ee1 || ! cc && ee2 ]
	];

logicalDenest[ pw_which ] := 						(* Denesting *)
    Module[{c, e, flow}, 
		{c, e} 	 = Transpose[Partition[List @@ pw, 2]];
		{cc, ee} = {Map[ logicalDenest, c ], Map[ logicalDenest, e ]};
		
        flow 	 = Table[And @@ (Not /@ Take[cc, k - 1]), {k, Length[c]}];
        Simplify @ (Or @@ Thread[And[flow, cc, ee]])
	];

logicalDenest[ e_?AtomQ ] := e;						(* General case *)

logicalDenest[ h_[ e___ ] ] := 						(* General case *)
    h[ Sequence @@ Map[ logicalDenest, {e} ] ];

(* ExtractNestedConditionals[ expr ] 
 *	Input: 
 *		expr	-- Expression involving UnitStep, Boole, If, Which
 *	Output: 
 *		Equivalent expression with all conditional expression nested 
 *		at the top of the expression. This effectively express 
 *		UnitStep and Boole in terms of if and which and uses distributive 
 *		properties of these functions. (Assuming no side effects.)
 * 
 *		Note: If you don't want if expression these can be removed by 
 *		the function removeIf.
 *		Note: If you don't want program flow to matter that can be
 *		removed by removeProgramFlow.
 *)

ExtractNestedConditionals[ expr_] :=
	NestedPWForm @ PiecewiseNormalize @ expr;

(* ExtractConditionals[ expr ] 
 *	Input: 
 *		expr	-- Expression involving UnitStep, Boole, If, Which
 *	Output: 
 *		Equivalent expression with one conditional expression 
 *		at the top of the expression. This effectively express 
 *		UnitStep, Boole, Sign and If in terms of Which and uses distributive 
 *		properties of these functions. (Assuming no side effects.)
 *)
ExtractConditionals[ expr_ ] :=
	flattenWhich @ removeIf @ ExtractNestedConditionals[ expr ]

(* Note: If[ False, e1 ] and Which[ False, e1 ] both autosimplify to Null *)
ExtractConditionals[ Null ] := which[ False, 0 ];
	
(* whichSimplify[ expr_ ]
 *
 * Applies inexpensive simplify steps to a piecewise expression as used at top level 
 *	for Integrate etc. 
 *
 *	which[ c1 -> e1, ..., cn -> en ]
 *
 *	which[ ..., cb -> eb , False -> ei, ca -> ea, ... ] ==> 
 *			which[ ..., cb -> eb, ca -> ea, ... ]
 *	which[ ..., cb -> eb , True -> ei, ca -> ea, ... ] ==> 
 *			which[ ..., cb -> eb ]
 *	-- Only applicable at top level and in this application ... 
 * 	which[  ..., cb -> eb , c -> 0, ca -> ea, ... ] ==> 
 *			which[ ..., cb -> eb, ca -> ea, ... ]
 *
 *)
whichSimplify[ expr_which ] :=
	Module[ { conds, exprs, remove, keep }, 
		{conds, exprs} = pwUnpack[ expr ];
		remove = Union[ Position[ conds, False ], Position[ exprs, 0 ] ];
		keep   = Complement[ Transpose[ {Range[ Length[ conds]]} ], remove ];
		pwPack[ { Extract[ conds, keep ], Extract[ exprs, keep ] } ]
	]

whichSimplify[ expr_ ] := expr;
	
(* CaseForm[ expr ]
 * 	Displays a box structure corresponding to the flattened form of an expression.
 *)
CaseForm[ expr_ ] :=
	whichForm[ ExtractConditionals[ expr ] ]

(* whichForm[ expr ]
 * Input:	
 *	expr	-- Expression containing UnitStep, Boole, If, Which
 * Output:	
 *	Nicely typeset expression where every conditional is treated 
 *	in a case form like Which. 
 *)
whichForm[ expr_ ] :=
	Module[ {cexpr = fromConditional[ expr ]},
		DisplayForm[
    		ReplaceRepeated[ cexpr, 
        		which[e__] :> whichBox[which[e]]
      		]
    	]
	]
	
whichBox[ expr_ ] :=
  	Module[ {c, e}, 
    	{c, e} = Transpose[Partition[ Apply[ List, expr ], 2 ]];
    	StyleBox[ 
      		RowBox[ 
        	{ 
          		"{", 
          		GridBox[
            		Table[ {c[[i]], "\[LongRightArrow]", e[[i]]}, {i, Length[c]}], 
            		ColumnAlignments -> Left
            	]
          	}
        	], 
      		AutoStyleOptions -> {"UnmatchedBracketStyle" -> None}, 
      		SpanMaxSize -> Infinity
      	]
    ]

(*---------------- Finite Piecewise Algrebraic Functions --------------------*)

(* Note: 	This adds experimental support for integrating the functions, 
 *			Abs, Sign, Min and Max in terms of piecewise functions. 
 *			Specifically the following rewrites are used which are valid over the 
 *			reals. 
 *
 *			Abs[ e ]			--> If[ e > 0, e, -e ]
 *			Sign[ e ]			--> If[ e > 0, 1, -1 ]
 *			Boole[ e ]			--> If[ e, 1, 0 ]
 *			UnitStep[e1,..,en] 	--> If[ e1 >=0 && ... && en >=0, 1, 0 ]
 *			Max[ e1, ..., en ]	--> Max[ e1, Max[ e2, ..., en] ] 
 *			Min[ e1, ..., en ]	--> Min[ e1, Min[ e2, ..., en] ]
 *			Max[ e1, e2 ]		--> If[ e1 > e2, e1, e2 ]
 *			Min[ e1, e2 ]		--> If[ e1 > e2, e2, e1 ]
 *
 *			The following direct translation turns out to be more effective.
 *
 *			Max[ e[1], ... , e[n] ] --> 
 *				Which[	e[1] >= e[2] && ... && e[1] >= e[n], e[1], 
 *						e[2] >= e[1] && ... && e[2] >= e[n], e[2], , 
 *						e[n] >= e[1] && ... && e[n] >= e[n - 1], e[n]]
 *
 *			And similarly for Min. 
 *			
 *)
				
FiniteConditionalFunctions[ expr_ ] :=
	Module[ {ineqs, exprs, nexprs},
		{ineqs, exprs} = pwUnpack[ expr ];
    	nexprs = exprs //.
		{
          Abs[e_] 			:> if[e > 0, e, -e],
          Sign[e_] 			:> which[e > 0, 1, e < 0, -1, e == 0, 0],
          Max[ef_, er__] 	:> max[ef, er], 
		  Min[ef_, er__] 	:> min[ef, er],
		  UnitStep[ e__ ] 	:> if[ And @@ (# >=0)& /@ {e}, 1, 0 ],
		  Boole[ e_ ] 		:> if[ e, 1, 0 ]
   		} /. {if -> If, which -> Which};
		ExtractConditionals[ pwPack[ {ineqs, nexprs} ] ]
	];

max[ e__ ] :=
  which @@
    Flatten @
      Table[
        { And @@ Thread[ Extract[ {e}, {i} ] > Drop[ {e}, {i} ]   ], 
          Extract[ {e}, {i} ]}, 
        {i, 1, Length[ {e} ]}
        ];

min[ e__ ] :=
  which @@
    Flatten @
      Table[
        { And @@ Thread[ Extract[ {e}, {i} ] < Drop[ {e}, {i} ]  ], 
          Extract[ {e}, {i} ]}, 
        {i, 1, Length[ {e} ]}
        ];

(*---------------- Infinite Piecewise Algebraic Functions -------------------*)

(* Note: This adds (experimental) support for the functions:
 *
 * 	Floor, Ceiling, Round, IntegerPart, FractionalPart, Mod, Quotient
 *
 *	As these functions are generally infinite piecewise functions, for 
 * 	this to work the range of values of their arguments have to be finite. 
 *
 *	The number of resulting cases may be very large. And hence we will need 
 *	exit in case there is a blowup.
 *)
$MaxPiecewiseCases = 10;

$infinitePiecewisePattern = _Floor | _Ceiling | _Round | _IntegerPart | _FractionalPart | _Mod | _Quotient;

(* emptySetQ[ ineqs, vars ]
 * 	Test whether a semi-algebraic set is empty or not. 
 *)
emptySetQ[ ineqs_, vars_ ] :=
	Experimental`ExistsRealQ[ ineqs, vars ] === False;

(*
Supremum[ e_, dom_, vars_ ] :=
	-Experimental`Infimum[ -e, dom, vars ];
*)

InfSup[ expr_, ineq_, vars_ ] :=
    Module[ {z, sineq,  nineq}, 
	(* Use strict inequalities -- performance optimization *)
      sineq = ineq /. {LessEqual -> Less, GreaterEqual -> Greater};
	  
	(* Use quantifier elimination maximally -- performance optimization. 
	   Note: The filtering operation is to be able to accept the new output format from Resolve *)
      nineq = Resolve[ Exists[ vars, Element[ Append[vars, z], Reals], sineq && z == expr ] ];
	  nineq = ReplaceAll[ nineq, _Element -> Sequence[] ];
	  
      {Floor[ Experimental`Infimum[ {z, nineq}, {z}]], Ceiling[ Experimental`Supremum[ {z, nineq}, {z} ] ]}
    ] /; FreeQ[ ineq, _Abs | _Max | _Min ];

(* Have to have this case because of a bug in Resolve *)
InfSup[ expr_, ineq_, vars_ ]:=
	{Floor[ Experimental`Infimum[ {expr, ineq}, vars]], Ceiling[ Experimental`Supremum[ {expr, ineq}, vars ] ]};

piecewiseRewrite[ Ceiling[ expr_ ], dom_, vars_ ] :=
    Module[ {min, max}, 
	  {min, max} = InfSup[ expr, dom, vars ];
	  Which[
	  	  	max - min === Infinity,
	  			(
	  			Message[ $MaxPiecewiseCases::"infinite", Ceiling[expr], dom ];
	  			Return[ $Failed ]
	  			),
	  		max - min > $MaxPiecewiseCases,
	  			(
	  			Message[ $MaxPiecewiseCases::"large", Ceiling[expr], dom, max - min, $MaxPiecewiseCases ];
	  			Return[ $Failed ]
	  			),
	  		!(IntegerQ[min] && IntegerQ[max]),
	  			Return[ $Failed ]
	  ];
	  pwPack[
	  	Transpose @
      	Table[ 
		  	{ min + i - 1 < expr <= min + i, min + i }, 
		  	{i, 1, max - min } 
		]
       ] /. which -> Which
      ] /; FreeQ[ expr, $infinitePiecewisePattern ];
	  
piecewiseRewrite[ Floor[ expr_ ], dom_ , vars_ ] :=
    Module[ {min, max}, 
	  {min, max} = InfSup[ expr, dom, vars ];
	  Which[
	  	max - min === Infinity,
			(
			Message[ $MaxPiecewiseCases::"infinite", Floor[expr], dom ];
			Return[ $Failed ]
			),
		max - min > $MaxPiecewiseCases,
			(
			Message[ $MaxPiecewiseCases::"large", Floor[expr], dom, max - min, $MaxPiecewiseCases ];
			Return[ $Failed ]
			),
		!(IntegerQ[min] && IntegerQ[max]),
			Return[ $Failed ]
	  ];
      pwPack[
	  	Transpose @
	  	Table[ 
			{ min + i - 1 <= expr < min + i, min + i - 1 }, 
			{i, 1, max - min } 
		]
      ] /. which -> Which
	] /; FreeQ[ expr, $infinitePiecewisePattern ];
	  
piecewiseRewrite[ Round[ expr_ ], dom_ , vars_ ] :=
    piecewiseRewrite[ Ceiling[ expr - 1/2 ], dom, vars ];
	
piecewiseRewrite[ IntegerPart[ expr_ ], dom_ , vars_ ] :=
    Module[ {min, max}, 
	  {min, max} = InfSup[ expr, dom, vars ];
	  Which[
	  	  	max - min === Infinity,
	  			(
	  			Message[ $MaxPiecewiseCases::"infinite", IntegerPart[expr], dom ];
	  			Return[ $Failed ]
	  			),
	  		max - min > $MaxPiecewiseCases,
	  			(
	  			Message[ $MaxPiecewiseCases::"large", IntegerPart[expr], dom, max - min, $MaxPiecewiseCases ];
	  			Return[ $Failed ]
	  			),
	  		!(IntegerQ[min] && IntegerQ[max]),
	  			Return[ $Failed ]
	  ];
      pwPack[
	  	Transpose @
	  	Table[ 
			If[ min + i - 1 >= 0, 
				{ min + i - 1 <= expr < min + i, min + i - 1 }, 
				{ min + i - 1 <= expr < min + i, min + i }
			],
			{i, 1, max - min } 
		]
      ] /. which -> Which
	] /; FreeQ[ expr, $infinitePiecewisePattern ];

piecewiseRewrite[ FractionalPart[ expr_ ], dom_, vars_  ] :=
    expr - piecewiseRewrite[ IntegerPart[ expr ], dom, vars ];
	
piecewiseRewrite[ Mod[ m_, n_ ], dom_, vars_ ] :=
	m - n piecewiseRewrite[ Floor[ m/n ], dom, vars ];

piecewiseRewrite[ Mod[ m_, n_, p_ ], dom_, vars_ ] :=
	m - n piecewiseRewrite[ Floor[ (m-p)/n ], dom, vars ];

piecewiseRewrite[ Quotient[ m_, n_], dom_, vars_ ] :=
	piecewiseRewrite[ Floor[ m/n ], dom, vars ];
	
piecewiseRewrite[ Quotient[ m_, n_, p_ ], dom_, vars_ ] :=
	piecewiseRewrite[ Floor[ (m-p)/n ], dom, vars ];

piecewiseRewrite[ e_, dom_, vars_ ] := e;

	
(* Note: This adds experimental support for eliminating the functions 
 *
 *	Floor, Ceiling, Round, IntegerPart, FractionalPart, Mod, Quotient
 *
 * from inequalities. 
 * 
 * Algorithm:
 *	0. Use the following rewrite rules to eliminate occurences of Round, 
 *	Quotient and Mod.
 *
 *	Mod: 			Mod[x,y] 		== x - y Floor[x/y]
 *					Mod[x,y,z]		== x - y Floor[(x-z)/y]
 *	Quotient:		Quotient[x,y]	== Floor[x/y]
 *					Quotient[x,y,z]	== Floor[(x-z)/y]
 *	Round:			Round[e]		== Ceiling[e-1/2]
 *
 *	1. Given an inequality r[ x1,.., xn, Floor[ e1 ], Ceiling[ e2 ] ], with 
 *		potentially nested occurrences of Floor, Ceiling, IntegerPart and 
 *		FractionalPart. Locate all such subexpressions: s1, s2, ... 
 *	2. Introduce new variables for all these expressions: y1, y2, ... 
 *		y1 <--> s1, y2 <--> s2, ... 
 *	3. Add inequalities for each of the subexpressions si according to the 
 *		inequalities: 
 *
 *	Floor: 			e-1 	<  Floor[e] 		<= e
 *  Ceiling:		e 		<= Ceiling[e] 		< e+1
 *	IntegerPart:	e-1 	<  IntegerPart[e] 	< e+1
 *	FractionalPart:	-1		<  FractionalPart[e]< 1
 *
 *	4. Replace all occurrences of si with their corresponding yi. This results 
 *		in a new logical combination of inequalies R[x1,..,xn,y1,..,ym].
 *	5. For each expression si = Floor[ e ] we can now compute the range of e through: 
 *			{Infimum[e, R, {x1,..,xn,y1,..,ym}], Supremum[e, R, {x1,..,xn,y1,..,ym}]}
 *		(See below for many performance enhancements.)
 *	6. Subsitute each si with its corresponding piecewise function in r and expand 
 *		the piecewise function.
 *
 * Example:
 *	Boole[ 1 < x + Floor[x] < 3 ]	==> 
 *			Boole[ 1 < x + Which[0 <= x < 1, 0, 1 <= x < 2, 1] < 3 ]
 *)
 
implicitPWExtract[ ineq_, vars_ ] :=
  Module[ {fexpr, fvars, y, sineq, fineq, nineq, objs, pws},
  	sineq = ineq /. 
		{
			Mod[x_, y_] 	:> x - y Floor[x/y],
			Mod[x_,y_,z_]	:> x - y Floor[(x-z)/y],
			Quotient[x_,y_]	:> Floor[x/y],
			Quotient[x_,y_,z_]	:> Floor[(x-z)/y],
			Round[e_]		:> Ceiling[e-1/2]
		};
  	fexpr = Union[ Cases[ sineq, _Floor | _Ceiling | _FractionalPart | _IntegerPart, {0, Infinity} ] ];
    fvars = Array[ y, {Length[fexpr]}];
    fineq = fexpr /. 
		{
          	Floor[ e_ ] 	:> e - 1 < Floor[e] <= e,
          	Ceiling[ e_ ] 	:> e <= Ceiling[e] < e + 1,
          	FractionalPart[ e_ ] :> -1 < FractionalPart[e] < 1,
          	IntegerPart[e_] :> e - 1 < IntegerPart[e] < e + 1
        };
    nineq = sineq && Apply[And, fineq] /. Thread[ fexpr -> fvars ];
	
	(* We don't need to continue if the resulting set is empty *)
	If[ emptySetQ[ nineq, Join[ vars, fvars ] ], Return[ False ] ];
	
    objs  = Map[ First, fexpr ] /. Thread[ fexpr -> fvars ];
	pws   = Table[ 
				fvars[[i]] -> 
				piecewiseRewrite[ Head[fexpr[[i]]][ objs[[i]] ], nineq, Join[ vars, fvars ] ], 
				{i, Length[ fexpr ]} 
			]/. Which -> which;
	((sineq /. Thread[ fexpr -> fvars ]) //. pws) /. which -> Which 
  ] /; !FreeQ[ ineq, $infinitePiecewisePattern ];

implicitPWExtract[ ineq_, vars_ ] := ineq;

explicitPWExtract[ expr_, ineq_, vars_ ] :=
	Module[ {fexpr, nexpr},
		If[ emptySetQ[ ineq, vars ], Return[ 0 ] ];
		fexpr = Select[ 
					Union[ Cases[ expr, $infinitePiecewisePattern, {0, Infinity} ] ],
					FreeQ[ #, $infinitePiecewisePattern, {1,Infinity} ]&
				];
		nexpr = Map[ piecewiseRewrite[ #, ineq, vars ]&, fexpr ];
		expr /. Thread[ fexpr -> nexpr ]
	] /; !FreeQ[ expr, $infinitePiecewisePattern ];

explicitPWExtract[ expr_, ineq_, vars_ ] := expr;
	
(* 
 * Note: Much of the complexity in this function stems from the fact that 
 *	many functions not yet having been extended to deal with piecewise functions. 
 *)
InfiniteConditionalFunctions[ pwexpr_, vars_ ] :=
	Module[ {nexpr, ineqs, exprs, nineqs, nexprs},
		nexpr = pwexpr;
		{ineqs, exprs} = pwUnpack[ nexpr ];
		
		(* Resolve inequalities *)
		nineqs = Map[ implicitPWExtract[ #, vars ]&, ineqs ];
		If[ FreeQ[ nineqs, $Failed ], 
			(
			nexpr = ExtractConditionals[ pwPack[ {nineqs, exprs} ] /. which->Which ];
			{ineqs, exprs} = pwUnpack[ nexpr ];
			),
			Return[ $Failed ]
		];
		
		(* Resolve expressions *)
		nexprs = Apply[ explicitPWExtract[ #2, #1, vars ]&, Transpose[ {ineqs, exprs} ], {1} ];
		If[ FreeQ[ nexprs, $Failed ], 
			(
			nexpr = ExtractConditionals[ pwPack[ {ineqs, nexprs} ] /. which -> Which ];
			If[ !FreeQ[ nexpr, $infinitePiecewisePattern ], 
				nexpr = InfiniteConditionalFunctions[ nexpr, vars ];
			];
			),
			Return[ $Failed ]
		];
		
		nexpr
	];


Protect[ Integrate, NIntegrate, Boole, Region ];
End[ (* "`Private`" *) ];
EndPackage[ (* "Calculus`Integration`" *) ];

Null
